// node pckgs
var fs = require('fs');
var path = require('path');
var util = require('util');
var querystring = require('querystring');
var spawn = require('child_process').spawn;

// 3rd party pckgs
var extend = require('extend');
var through = require('through');

function nocache(response)
{
    response.setHeader('expires', 'Sat, 10 Feb 1990 00:00:00 GMT');
    response.setHeader('pragma', 'no-cache');
    response.setHeader('cache-control', 'no-cache, max-age=0, must-revalidate');
};

function pack(input)
{
    input = util.format('# service=%s\n', input);

    var n = (4 + input.length).toString(16);
    return Array(4 - n.length + 1).join('0') + n + input;
};

function figureService(request)
{
    var result = null;
    var qmIndex = request.originalUrl.indexOf('?');

    if(qmIndex > -1)
    {
        result = querystring.parse(request.originalUrl.substr(qmIndex + 1)).service;
    }
    else
    {
        var parts = request.originalUrl.split('/');
        result = parts[parts.length - 1];
    }

    return result;
};

var encodings = {
    'gzip': function() {return zlib.createGunzip();},
    'deflate': function() {return zlib.createDeflate();}
};

module.exports = function(opts)
{
    var options = extend({
        folder: '',

        resolve: function(request, done)
        {
            if(request.params.repository == null)
            {
                done(new Error('Parameter "repository" not defined.'), null);
                return;
            }

            var fullpath = path.join(options.folder, request.params.repository);

            fs.stat(fullpath, function(error, stats)
            {
                done.apply(null, error == null ? [null, fullpath] : [new Error('Repository directory not found.'), null])
            });
        }
    }, opts);

    return function(request, response, next)
    {
        if(request.method !== 'GET' && request.method !== 'POST')
        {
            next();
            return;
        }

        var service = figureService(request);

        response.setHeader('connection', 'close');

        options.resolve.apply(options, [request, function(error, repoPath)
        {
            if(error != null)
            {
                response.statusCode = 404;
                response.setHeader('Content-Type', 'text/plain');
                response.end('Not found.');

                return;
            }

            if(request.method === 'GET')
            {
                response.setHeader('Content-Type', util.format('application/x-%s-advertisement', service));

                nocache(response);

                response.write(pack(service));
                response.write('0000');

                var cmd = spawn(service, ['--stateless-rpc', '--advertise-refs', repoPath]);

                cmd.stdout.pipe(response);
            }

            if(request.method === 'POST')
            {
                response.setHeader('content-type', util.format('application/x-%s-result', service));
                nocache(response);

                var stream = through();
                var buffer = through().pause();

                var decoder = encodings[request.headers['content-encoding']];

                (decoder == null ?
                request.pipe(stream) :
                request.pipe(decoder()).pipe(stream)).pipe(buffer);

                stream.once('data', function(buf)
                {
                    var data = buf + '';

                    var infos = data.split('\n')[0].split(' ');

                    var ps = spawn(service, ['--stateless-rpc', repoPath]);

                    ps.stdout.pipe(response);

                    buffer.pipe(ps.stdin);
                    buffer.resume();
                });
            }
        }]);
    };
};
